#ifndef ELEMENT_H
#define ELEMENT_H

#include "gmc-common_global.h"

#include "treenode.h"

#include <QString>
#include <QIcon>

#include "gmccommon.h"

class QAction;

class Module;
class Element;
class ElementData;
class ElementWidget;

class GMC_COMMON_SHARED_EXPORT ElementRootTreeNode : public TreeNode
{

    Q_OBJECT

public:

    enum ElementColumnsType { NameColumn
                            , _CountColumns };

    ElementRootTreeNode( QObject * /*parent*/ = 0 );
    virtual ~ElementRootTreeNode();

    virtual void insert( int /*start*/, const QList< TreeNode *> & /*nodes*/ );
    virtual void removeAt( int /*start*/, int /*count*/ = 1 );

    virtual int columnCount() const;
    virtual QVariant headerData( int /*section*/
                               , Qt::Orientation /*orientation*/
                               , int /*role*/ = Qt::DisplayRole ) const;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ContextOperation
{

public:

    enum ContextType { ElementContext
                     , ElementWidgetContext
                     , _CountColumns };

public:

    ContextOperation( ContextOperation::ContextType /*contextType*/
                    , Module &/*module*/
                    , bool ( * /*isEditable*/ )( const Module & ) );
    virtual ~ContextOperation();

    Module &module() const;//protected:

    const QString &elementType() const;
    ContextOperation::ContextType contextType() const;

    virtual qulonglong availableOperations() const;
    virtual void performOperation( OperationType /*operation*/
                                 , ElementData * /*elementData*/ = 0
                                 , int /*depth*/ = 0 );

    virtual QString caption() const = 0;

    Element *elementContext();
    ElementWidget *elementWidgetContext();

protected:

    bool isEditable() const;

public:

    static const QString undefinedElementType;
    static const QString allElementType;

protected:

    Module &_module;
    bool ( * _isEditable )( const Module & );
    QString _elementType;

private:

    ContextType _contextType;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT Element
: public TreeNode
, public ContextOperation
{

    Q_OBJECT


public:

    static const QIcon &defaultIcon();

protected:

    static const QString &defaultQuestionForRemove();

public:

    Element( Module & /*module*/
           , bool ( * /*isEditable*/ )( const Module & ) );
    virtual ~Element();

    virtual void insert( int /*start*/, const QList< TreeNode *> & /*nodes*/ );
    virtual void removeAt( int /*start*/, int /*count*/ = 1 );

    const QString &groupElementType() const;
    bool isPersistent() const;

    ElementData *elementData() const;
    void setElementData( ElementData * /*elementData*/ );

    virtual ElementWidget *elementWidget( ElementWidgetRole /*role*/ );
    QList< QAction * > elementActions() const;

    virtual int columnCount() const;
    virtual QVariant data( int /*column*/, int /*role*/ = Qt::DisplayRole ) const;
    virtual bool canFetchMore() const;
    virtual void fetchMore();

    const QString &title() const;
    void setTitle( const QString & /*title*/);

    virtual int displayChildrenVariant() const;
    virtual QList< QString > displayChildrenVariants() const;

    virtual const QString& questionForRemove() const;

protected:

    QString _groupElementType;

    ElementData *_elementData;

    bool _requireLoadChildrens;

    QString _title;

    int _displayChildrenVariant;
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT LoginElement : public Element
{

    Q_OBJECT

public:

    LoginElement( Module & /*module*/
                , bool ( * /*isEditable*/ )( const Module & ) );
    virtual ~LoginElement();

    virtual qulonglong availableOperations() const;
    virtual void performOperation( OperationType /*operation*/
                                 , ElementData * /*elementData*/ = 0
                                 , int /*depth*/ = 0 );

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ServiceElement : public Element
{

    Q_OBJECT

protected:

    static const QIcon &startIcon();
    static const QIcon &stopIcon();

public:

    ServiceElement( Module & /*module*/
                  , bool ( * /*isEditable*/ )( const Module & ) );
    virtual ~ServiceElement();

    virtual QVariant data( int /*column*/, int /*role*/ = Qt::DisplayRole ) const;
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ContainerElement : public Element
{

    Q_OBJECT

public:

    ContainerElement( Module & /*module*/
                    , bool ( * /*isEditable*/ )( const Module & ) );
    virtual ~ContainerElement();

    virtual qulonglong availableOperations() const;
    virtual void performOperation( OperationType /*operation*/
                                 , ElementData * /*elementData*/ = 0
                                 , int /*depth*/ = 0 );

    void resetRequireLoad();
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT SupplementLoginMixin
{

public:

    SupplementLoginMixin( Element * /*element*/ );
    ~SupplementLoginMixin();

protected:

    int treeNodeSpecialFlags( int /*specialFlags*/ ) const;

protected:

    Element *_element;
};

//******************************************************************************

//public:

class GMC_COMMON_SHARED_EXPORT OrdinaryElement : public Element

{
    Q_OBJECT


public:

    OrdinaryElement( Module & /*module*/
                   , bool ( * /*isEditable*/ )( const Module & ) );
    virtual ~OrdinaryElement();

    virtual qulonglong availableOperations() const;
    virtual void performOperation( OperationType /*operation*/
                                 , ElementData * /*elementData*/ = 0
                                 , int /*depth*/ = 0 );

    virtual int displayChildrenVariant() const;
    virtual QList< QString > displayChildrenVariants() const;

};

//******************************************************************************

//public:

class GMC_COMMON_SHARED_EXPORT SupplementElement : public Element

{
    Q_OBJECT


public:

    SupplementElement( Module & /*module*/
                     , bool ( * /*isEditable*/ )( const Module & ) );
    virtual ~SupplementElement();

    virtual int treeNodeSpecialFlags() const;

    virtual qulonglong availableOperations() const;
    virtual void performOperation( OperationType /*operation*/
                                 , ElementData * /*elementData*/ = 0
                                 , int /*depth*/ = 0 );

};

//******************************************************************************

#endif // ELEMENT_H
