#ifndef ELEMENTDATA_H
#define ELEMENTDATA_H

#include "gmc-common_global.h"

#include <QString>
#include <QSet>
#include <QMap>

#include <treenode.h>

class GMC_COMMON_SHARED_EXPORT ElementData : public QMap< QString, QVariant >
{

public:

    ElementData();
    ElementData( const ElementData & /*elementData*/ );
    virtual ~ElementData();

    const ElementData & importFrom( const ElementData & /*elementData*/ );

    const QString name() const;
    void setName( const QString & /*name*/ );

    virtual void configureParams();
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ChangeDisplayElementData : public ElementData
{

public:

    ChangeDisplayElementData();
    ChangeDisplayElementData( int /*displayType*/ );
    ChangeDisplayElementData( const ChangeDisplayElementData & /*elementData*/ );
    virtual ~ChangeDisplayElementData();

    const ChangeDisplayElementData & operator=( const ChangeDisplayElementData & /*elementData*/ );

    int displayType() const;
    void setDisplayType( int /*displayType*/ );

protected:

    int _displayType;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ServiceLightElementData : public ElementData
{

public:

    ServiceLightElementData();
    ServiceLightElementData( const ServiceLightElementData & );
    virtual ~ServiceLightElementData();

    const ServiceLightElementData& operator=( const ServiceLightElementData & );

    bool state() const;
    const QString & textState() const;

    void setState( bool /*state*/, const QString & /*textState*/ = QString() );

protected:

    bool _state;
    QString _textState;
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ServiceHeavyElementData : public ServiceLightElementData
{

public:

    ServiceHeavyElementData();
    ServiceHeavyElementData( const ServiceHeavyElementData & );      // _data not copied

    virtual ~ServiceHeavyElementData();                              // _data deleted

    const ServiceHeavyElementData &operator=( const ServiceHeavyElementData & ); // _data not copied

    QList< TreeNode * > & data();

    const QString &configFile() const;
    void setConfigFile(const QString &configFile);

    const QString &configData() const;
    void setConfigData(const QString &configData);

protected:

    QList< TreeNode * > _data;
    QString _configData;
    QString _configFile;

};

//******************************************************************************


//class GMC_COMMON_SHARED_EXPORT LoginElementData : public ElementData
//{

//public:

//    LoginElementData();
//    LoginElementData( const LoginElementData & /*elementData*/ );
//    virtual ~LoginElementData();

//    const LoginElementData &operator =( const LoginElementData & /*elementData*/ );

//    bool isLogin() const;
//    void setLogin ( bool /*isLogin*/ );

//    const QString &user() const;
//    void setUser(const QString &user);

//    const QString &password() const;
//    void setPassword(const QString &password);

//protected:

//    bool _isLogin;
//    QString _user;
//    QString _password;

//};

//******************************************************************************


class GMC_COMMON_SHARED_EXPORT ServiceConfigureHeavyElementData : public ServiceLightElementData
{
public:

    ServiceConfigureHeavyElementData();
    ServiceConfigureHeavyElementData( const ServiceConfigureHeavyElementData & /*elementData*/ );
    virtual ~ServiceConfigureHeavyElementData();

    const ServiceConfigureHeavyElementData &operator=( const ServiceConfigureHeavyElementData & /*elementData*/ );

    QStringList &scriptModeCommands();
    QStringList &headers();
    QStringList &descriptions();
    QMultiMap< QString, QString > &tasks();

    QSet< QString > &checkedItems();
    QSet< QString > &uncheckedItems();
    QSet< QPair< QString, QString > > &incompatibleModes();

    QString password() const;
    void setPassword(const QString &password);

    QString user() const;
    void setUser(const QString &user);

protected:
    QStringList _scriptModeCommands;
    QStringList _headers;
    QStringList _descriptions;
    QMultiMap< QString, QString> _tasks;
    QString _user;
    QString _password;

    QSet< QString > _checkedItems;
    QSet< QString > _uncheckedItems;
    QSet< QPair< QString, QString > > _incompatibleModes;
};

//******************************************************************************

#endif // ELEMENTDATA_H
