#ifndef ELEMENTWIDGET_H
#define ELEMENTWIDGET_H

#include "gmc-common_global.h"

#include <QWidget>

#include "gmccommon.h"
#include "element.h"

class QModelIndex;

class Module;
class ElementData;
class ElementModel;
class ListWidgetProxyModel;

class GMC_COMMON_SHARED_EXPORT ElementWidget
: public QWidget
, public ContextOperation
{

    Q_OBJECT

signals:

    void elementContextOperationChanged( Element * );

public:

    static bool priorityLessThen( const ElementWidget * const/*left*/
                                , const ElementWidget * const/*right*/ );

    ElementWidget( Module & /*module*/
                 , bool ( * /*isEditable*/ )( const Module & )
                 , QWidget * /*parent*/ = 0 );
    virtual ~ElementWidget();

    virtual qulonglong availableOperations() const;
    virtual void performOperation( OperationType /*operation*/
                                 , ElementData * /*elementData*/ = 0
                                 , int /*depth*/ = 0 );

    const QString & supplementElementWidgetType() const;
    virtual void setAlternativeExpected( int /*state*/ );

    Element *element() const;
    virtual void setElement( Element * /*element*/, ElementWidgetRole /*role*/ );

    ElementWidgetRole role() const;

    virtual void read();

    virtual QString caption() const;

protected:

    virtual void showEvent( QShowEvent * /*event*/ );

protected:

    QString _supplementElementWidgetType;
    int _priority;

    Element *_element;
    ElementWidgetRole _role;
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ViewEditElementWidget : public ElementWidget
{

    Q_OBJECT

signals:

    void contentChanged( int );

public:

    enum AvailibleWidgetActions{ ApplyAction    = 0x1
                               , CancelAction   = 0x2 };

    ViewEditElementWidget( Module & /*module*/
                         , bool ( * /*isEditable*/ )( const Module & )
                         , QWidget *parent = 0 );
    virtual ~ViewEditElementWidget();

    virtual qulonglong availableOperations() const;
    virtual void performOperation( OperationType /*operation*/
                                 , ElementData * /*elementData*/ = 0
                                 , int /*depth*/ = 0 );

    virtual void setElement( Element * /*element*/, ElementWidgetRole /*role*/ );

    virtual void read();

    bool isChanged() const;
    void setUnchanged();

    virtual void beforeCreation();
    virtual ElementData *exportElementData() const;
    virtual void afterSuccessfulRead( Element * , ElementData * );
    virtual void afterSuccessfulImport( ElementData * );
    virtual void afterSuccessfulUpdate();

    virtual void applyContentChanges();

protected:

    virtual bool linkExternalElements();
    virtual void reset();

    virtual bool isContentValid() const;
    virtual void blockEditWidget( bool ); //enable/disable control & block/unblocksignal control
//    void queryImportFrom( ElementData * ) const;

protected slots:

    void changeContent();

    void setTreeNodeCurrent( TreeNode * );
    void noSetTreeNodeCurrent();

protected:

    bool _blocked;
    bool _changed;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ListElementWidget : public ElementWidget
{

    Q_OBJECT

public:

    ListElementWidget( const QString & /*elementWidgetType*/
                     , Module & /*module*/
                     , bool ( * /*isEditable*/ )( const Module & )
                     , QWidget * /*parent*/ = 0 );

    virtual ~ListElementWidget();

protected:

    ElementModel *_elementModel;
    ListWidgetProxyModel *_elementProxyModel;

};

//******************************************************************************

namespace Ui
{
    class NodesElementWidget;
}

class GMC_COMMON_SHARED_EXPORT NodesElementWidget : public ListElementWidget
{
    Q_OBJECT

public:

    NodesElementWidget( const QString & /*elementWidgetType*/
                      , Module & /*module*/
                      , bool ( * /*isEditable*/ )( const Module & )
                      , QWidget * /*parent*/ = 0 );
    ~NodesElementWidget();

    virtual void read();

public slots:

    void choiceElement( const QModelIndex & /*index*/ );

protected slots:

    void customContextMenuRequest( const QPoint & );

protected:

    Ui::NodesElementWidget *_ui;

};

//******************************************************************************

namespace Ui
{
    class ElementsElementWidget;
}

class GMC_COMMON_SHARED_EXPORT ElementsElementWidget : public ListElementWidget
{

    Q_OBJECT

public:

    ElementsElementWidget( const QString &/*elementWidgetType*/
                         , Module & /*module*/
                         , bool ( * /*isEditable*/ )( const Module & )
                         , QWidget * /*parent*/ = 0 );
    virtual ~ElementsElementWidget();

    virtual void read();

protected:

    void loadSettings();
    void saveSettings() const;

public slots:

    void changeFilter();
    void choiceElement( const QModelIndex & /*index*/ );

protected slots:

    void customContextMenuRequest( const QPoint & );

protected:

    Ui::ElementsElementWidget *_ui;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT SupplementElementsElementWidget : public ElementsElementWidget
{

    Q_OBJECT

public:

    SupplementElementsElementWidget( const QString &/*elementWidgetType*/
                                   , const QString & /*title*/
                                   , Module & /*module*/
                                   , bool ( * /*isEditable*/ )( const Module & )
                                   , QWidget * /*parent*/ = 0 );
    virtual ~SupplementElementsElementWidget();

};

#endif // ELEMENTWIDGET_H
