#ifndef MISCMODEL_H
#define MISCMODEL_H

#include "gmc-common_global.h"

#include <QAbstractTableModel>

#include <QList>
#include <QSet>

class GMC_COMMON_SHARED_EXPORT MiscModel : public QAbstractTableModel
{

    Q_OBJECT

public:

    enum InfoColumnsType { DataColumn
                         , _CountColumns };

    MiscModel( QObject * /*parent*/ = 0 );
    virtual ~MiscModel();

    int rowCount( const QModelIndex & /*parentIndex*/ = QModelIndex() ) const;
    int columnCount( const QModelIndex & /*parentIndex*/ = QModelIndex() ) const;

    virtual QVariant data( const QModelIndex & /*index*/, int /*role*/ = Qt::DisplayRole ) const;
    virtual Qt::ItemFlags flags( const QModelIndex & /*index*/ ) const;
    virtual QVariant headerData( int /*section*/, Qt::Orientation /*orientation*/, int /*role*/ = Qt::DisplayRole) const;

    const QList< QString > & data() const;
    virtual void setData( const QList< QString>  & /*data*/ );
    virtual void clear();

protected:

    QList< QString > _data;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT CheckMiscModel : public MiscModel
{

    Q_OBJECT

public:

    CheckMiscModel( QObject* /*parent*/ = 0 );
    virtual ~CheckMiscModel();

    virtual QVariant data( const QModelIndex & /*index*/, int /*role*/ = Qt::DisplayRole ) const;
    virtual bool setData( const QModelIndex & /*index*/, const QVariant & /*value*/, int /*role*/ = Qt::EditRole );
    virtual Qt::ItemFlags flags( const QModelIndex & /*index*/ ) const;

    virtual void setData( const QList< QString>  & /*data*/ );
    virtual void clear();
    void setDataChecked( const QList< QString>  & /*data*/, const QSet< QString > & /*checkedData*/ );
    void setDataUnchecked( const QList< QString>  & /*data*/, const QSet< QString > & /*uncheckedData*/ );


    const QSet<QString> & dataToAdd() const;
    const QSet<QString> & dataToRemove() const;

    QSet< QString > checkedData() const;
    QSet< QString > uncheckedData() const;

protected:

    QList< bool > _checkData;
    QSet< QString > _dataToAdd;
    QSet< QString > _dataToRemove;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ConfigureCheckModel : public CheckMiscModel
{

    Q_OBJECT

public:

    ConfigureCheckModel( QObject * /*parent*/ = 0 );
    virtual ~ConfigureCheckModel();

    virtual QVariant data( const QModelIndex &/*index*/, int /*role*/ = Qt::DisplayRole ) const;
    virtual bool setData( const QModelIndex & /*index*/, const QVariant & /*value*/, int /*role*/ );
    virtual Qt::ItemFlags flags( const QModelIndex & /*index*/ ) const;

    QStringList descriptions() const;
    void setDescriptions( const QStringList &descriptions );

    const QSet<int> &inCmpatibleModes() const;
    void setInCmpatibleModes(const QSet<int> &inCmpatibleModes );

protected:
    bool checkCompatible( int, int ) const;
protected:

    QSet< int > _inCompatibleModes;

    QStringList _descriptions;
};


#endif // MiscModel_H
