#ifndef MODULE_H
#define MODULE_H

#include "gmc-common_global.h"

#include <QList>
#include <QSet>
#include <QMap>

#include "gmccommon.h"
#include "treenode.h"
#include "element.h"

class ElementData;
class ElementWidget;
class QAction;

class GMC_COMMON_SHARED_EXPORT ModuleRootTreeNode : public TreeNode
{

    Q_OBJECT

public:

    enum ModuleColumnsType { IdColumn
                           , ParanoidColumn
                           , DescriptionColumn
                           , _CountColumns };

    ModuleRootTreeNode( QObject * /*parent*/ = 0 );
    virtual ~ModuleRootTreeNode();

    virtual int columnCount() const;
    virtual QVariant headerData( int /*section*/
                               , Qt::Orientation /*orientation*/
                               , int /*role*/ = Qt::DisplayRole ) const;
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT Module : public TreeNode
{
    Q_OBJECT

signals:

    void contentElementWidgetChanged( int );
    void customContextMenuRequested( const QPoint & );

public:

    static bool defaultTrue( const Module & /*module*/ );
    static bool defaultFalse( const Module & /*module*/ );
    static bool isAdministrator( const Module & /*module*/ );
    static bool isAdministratorHierarchy( const Module & /*module*/ );
    static bool isMacAdministratorHierarchy( const Module & /*module*/ );

    Module( const QString & /*id*/
          , QObject * /*parent*/ = 0
          , bool /*paranoid*/ = false );
    virtual ~Module();

    virtual QVariant data( int /*column*/, int /*role*/ = Qt::DisplayRole ) const;
    virtual bool setData( int /*column*/, const QVariant & /*value*/, int /*role*/ = Qt::EditRole );
    virtual Qt::ItemFlags flags( int /*column*/ ) const;

    virtual bool canFetchMore() const;
    virtual void fetchMore();

    ElementData *moduleData() const;

    QVariant getData( const QString & /*param*/ ) const;
    QVariant getDataHierarchy( const QString & /*param*/ ) const;

    const QList< TreeNode * > & elementNodes() const;
    Element * element( const QString & /*id*/ ) const;

    virtual void performOperation( ContextOperation * /*context*/
                                 , OperationType /*operation*/
                                 , ElementData * /*elementData*/ = 0
                                 , int /*depth*/ = 0 );

    ElementWidget * elementWidget( const QString & /*elementType*/ ) const;

    QList< QAction * > elementActions( const QString & /*elementType*/ ) const;

protected:

    void insertElementWidget( ElementWidget * /*elementWidget*/ );
    void connectElementWidgets();
    void resetConteinerElement();

    virtual void loadSettings();
    virtual void saveSettings() const;

protected:

    ElementData *_elementData;

    QList< TreeNode * > _elements;

    QMap< QString, ElementWidget * > _elementWidgets;
    QMap< QString, QList< QAction * > > _elementActions;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT AcceptorModule : public Module
{

    Q_OBJECT

public:

    AcceptorModule( const QString & /*id*/
                  , int /*expectedDepth*/
                  , const QString & /*expectedParentId*/
                  , const QString & /*description*/
                  , QObject * /*parent*/ = 0 );
    virtual ~AcceptorModule();

    Element * elementFromDonor( const QString & /*id*/ ) const;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT SupplementModule : public Module
{

    Q_OBJECT

public:

    SupplementModule( const QString & /*id*/
                    , int /*expectedDepth*/
                    , const QString & /*expectedParentId*/
                    , const QString & /*description*/
                    , const QString & /*supplementElementType*/
                    , Module * /*supplementModule*/
                    , QObject * /*parent*/ = 0 );
    virtual ~SupplementModule();

    virtual void performOperation( ContextOperation * /*context*/
                                 , OperationType /*operation*/
                                 , ElementData * /*elementData*/ = 0
                                 , int /*depth*/ = 0 );

protected:

    Module *_supplementModule;
    const QString _supplementElementType;

};

//******************************************************************************

#endif // MODULE_H
