#ifndef TASKOBSERVER_H
#define TASKOBSERVER_H

#include "gmc-common_global.h"

#include <QObject>

#include <QPair>
#include <QList>
#include <QMap>

#include "task.h"

class QTimer;
class QAbstractItemModel;

class AbstractTask;
class TaskObserverModel;
class BlockDialog;
class TaskObserverDialog;
class TaskPerformedEvent;

class GMC_COMMON_SHARED_EXPORT TaskObserver : public QObject
{

    Q_OBJECT

    friend class TaskObserverModel;
    friend class TaskObserverDealog;

signals:

    void tasksAboutToBeInserted( int /*start*/, int /*end*/ );
    void tasksAboutToBeRemoved( int /*start*/, int /*end*/ );
    void tasksInserted( int /*start*/, int /*end*/ );
    void tasksRemoved( int /*start*/, int /*end*/ );
    void taskDataChanged( int /*index*/ );

public:

    static TaskObserver* instance( QObject * /*parent*/ = 0 );

    virtual ~TaskObserver();

    QAbstractItemModel *model() const;

    void performTask( AbstractTask * /*task*/ );

public slots:

    void openTaskObserveDialog();

    void clearTasks();

protected :

    TaskObserver( QObject * /*parent*/ = 0 );

    virtual bool event( QEvent * /*event*/ );
    virtual void taskPerformedEvent( TaskPerformedEvent * /*event*/ );

protected:

    static const int _delay;
    static TaskObserver *_instance;

    int _unknownPerformedTask;
    int _failPerformedTask;

    QMap< AbstractTask *, int > _tasks;
    QList< QPair< TaskDescription, AbstractTask * > > _taskDescriptions;

    QTimer *_timer;
    TaskObserverModel *_model;

    BlockDialog *_blockDialog;
    TaskObserverDialog *_taskObserverDialog;

};

#endif // TASKOBSERVER_H
