#ifndef TREENODE_H
#define TREENODE_H

#include "gmc-common_global.h"

#include <QObject>

#include <QString>
#include <QVariant>
#include <QList>


class GMC_COMMON_SHARED_EXPORT TreeNode : public QObject
{

    Q_OBJECT

signals:

    void nodesAboutToBeInserted( TreeNode * /*parent*/, int /*start*/, int /*end*/ );
    void nodesAboutToBeRemoved( TreeNode * /*parent*/, int /*start*/, int /*end*/ );
    void nodesInserted();
    void nodesRemoved();
    void nodeDataChanged( TreeNode * /*node*/ );
    void nodeChildsDataChanged( TreeNode * /*node*/ );
    void setCurrentNode( TreeNode * );
    void expandCurrentNode( TreeNode * );

public:

    enum SpecialTreeNodeFlags { NoSpecialFlag = 0x0     // ordinary tree node
                              , InvisibleFlag = 0x1     // false == TreeNodeInvisibleSortFilterProxyModel::filterAcceptsRow()
                              , SelectableFlag = 0x2    // true == TreeNodeSelectableSortFilterProxyModel::filterAcceptsRow()
                              , HideChildFlag = 0x4
                              };

    static bool expectedDepthLessThen( TreeNode * /*left*/, TreeNode * /*right*/ );

    TreeNode( QObject * /*parent*/ = 0 );
    TreeNode( int /*expectedDepth*/
            , const QString & /*expectedParentId*/
            , QObject * /*parent*/ = 0 );

    virtual ~TreeNode();

    TreeNode *parentNode() const;

    int childNodesCount() const;
    const QList< TreeNode *> &childNodes() const;
    QList< TreeNode * > allChildNodes() const;

    void append( TreeNode * /*node*/);                                      // !!! node->setParentNode( this );
    void append( const QList< TreeNode *> & /*nodes*/ );                    // !!! node->setParentNode( this );
    void clear();                                                           // !!! node->setParentNode( 0 );
    void clearDelete();                                                     // delete only direct childs
    void clearDeleteAll();                                                  // delete all childs

    bool contains( TreeNode * /*node*/ ) const;
    int indexOf( TreeNode * /*node*/, int /*from*/ = 0 ) const;

    void insert( int /*start*/, TreeNode * /*node*/ );                              // !!! node->setParentNode( this );
    virtual void insert( int /*start*/, const QList< TreeNode *> & /*nodes*/ );     // !!! node->setParentNode( this );
    virtual void removeAt( int /*start*/, int /*count*/ = 1 );                      // !!! node->setParentNode( 0 );
    bool removeOne( TreeNode * /*node*/ );                                          // !!! node->setParentNode( 0 );

    const QString &id() const;
    void setId( const QString & /*id*/ );

    const QString &expectedParentId() const;
    int expectedDepth() const;
    virtual void setAlternativeExpected( int /*state*/ );

    void loadTreeFromList( const QList< TreeNode * > & /*nodeList*/ );
    virtual void afterLoad();

    virtual int treeNodeSpecialFlags() const;

    virtual int columnCount() const;
    virtual QVariant headerData( int /*section*/
                               , Qt::Orientation /*orientation*/
                               , int /*role*/ = Qt::DisplayRole ) const;
    virtual QVariant data( int /*column*/, int /*role*/ = Qt::DisplayRole ) const;
    virtual bool setData( int /*column*/, const QVariant & /*value*/, int /*role*/ = Qt::EditRole );
    virtual Qt::ItemFlags flags( int /*column*/ ) const;

    virtual bool canFetchMore() const;
    virtual void fetchMore();

    void emitNodeDataChanged();
    void emitNodeChildsDataChanged();
    void emitSetCurrentNode();
    void emitExpandCurrentNode();

protected:

    void reset();

    void setParentNode( TreeNode * /*node*/ );
    void connectChildSignals( TreeNode * /*node*/ );
    void disconnectChildSignals( TreeNode * /*node*/ );

protected:

    TreeNode *_parentNode;
    QList< TreeNode * > _childNodes;

    QString _id;
    QString _expectedParentId;
    int _expectedDepth;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ConfRootTreeNode : public TreeNode
{

    Q_OBJECT

public:

    enum ConfColumnsType { NameColumn
                         , ValueColumn
                         , _CountColumns };

    ConfRootTreeNode( QObject * /*parent*/ = 0 );
    virtual ~ConfRootTreeNode();

    virtual int columnCount() const;
    virtual QVariant headerData( int /*section*/
                               , Qt::Orientation /*orientation*/
                               , int /*role*/ = Qt::DisplayRole ) const;
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ConfTreeNode : public TreeNode
{

    Q_OBJECT

public:

    ConfTreeNode( const QString & /*name*/
                , const QString & /*value*/
                , QObject * /*parent*/ = 0 );
    virtual ~ConfTreeNode();

    virtual QVariant data( int /*column*/, int /*role*/ = Qt::DisplayRole ) const;

protected:

    QString _name;
    QString _value;

};

#endif // TREENODE_H
