#ifndef TREENODEMODEL_H
#define TREENODEMODEL_H

#include "gmc-common_global.h"

#include <QAbstractItemModel>
#include <QIdentityProxyModel>
#include <QSortFilterProxyModel>

#include <QSet>

#include "treenodemodelextension.h"

class GMC_COMMON_SHARED_EXPORT TreeNodeModel
: public QAbstractItemModel
, virtual public TreeNodeModelExtension
{

    Q_OBJECT

public:

    TreeNodeModel( QObject * /*parent*/ = 0);
    TreeNodeModel( TreeNode * /*rootNode*/, QObject * /*parent*/ = 0);
    virtual ~TreeNodeModel();

    void setRootNode( TreeNode * /*rootNode*/ );

    virtual QVariant data( const QModelIndex & /*index*/
                         , int /*role*/ = Qt::DisplayRole ) const;
    virtual bool setData( const QModelIndex & /*index*/
                        , const QVariant & /*value*/
                        , int /*role*/ = Qt::EditRole );
    virtual Qt::ItemFlags flags(const QModelIndex & /*index*/ ) const;
    virtual QModelIndex index( int /*row*/
                             , int /*column*/
                             , const QModelIndex & /*parentIndex*/ = QModelIndex() ) const;

    virtual QModelIndex parent( const QModelIndex & /*index*/ ) const;
    virtual int rowCount( const QModelIndex & /*index*/ = QModelIndex() ) const;
    virtual int columnCount( const QModelIndex & /*index*/ = QModelIndex() ) const;
    virtual QVariant headerData( int /*section*/
                               , Qt::Orientation /*orientation*/
                               , int /*role*/ = Qt::DisplayRole ) const;

    virtual bool canFetchMore(const QModelIndex & /*parent*/ ) const;
    virtual void fetchMore(const QModelIndex & /*parent*/);
    virtual bool hasChildren(const QModelIndex & /*parent*/ ) const;

    virtual TreeNode *rootNode() const;

    virtual TreeNode * toTreeNode( const QModelIndex & /*index*/ ) const;
    virtual TreeNode * toTreeNode( const QItemSelectionRange & /*selectionRange*/ ) const;
    virtual TreeNode * toTreeNode( const QItemSelection & /*selection*/ ) const;

    virtual QModelIndex treeNodeToIndex( TreeNode * /*node*/ ) const;

    const QList< TreeNode * > &extraRootChildNodes() const;
    void setExtraRootChildNodes( const QList<TreeNode *> &/*extraRootChildNodes*/ );
    void deleteExtraRootChildNodes();

protected:

    void connectVertexNode( TreeNode * /*node*/ );
    void disconnectVertexNode( TreeNode * /*node*/ );

public slots:

    void nodesAboutToBeReset();
    void nodesReset();

protected slots:

    void nodesAboutToBeInserted( TreeNode * /*parent*/, int /*start*/, int /*end*/ );
    void nodesAboutToBeRemoved( TreeNode * /*parent*/, int /*start*/, int /*end*/ );
    void nodesInserted();
    void nodesRemoved();
    void nodeDataChanged( TreeNode * /*node*/ );
    void nodeChildsDataChanged( TreeNode * /*node*/ );

protected:

    TreeNode *_rootNode;
    QList< TreeNode * > _extraRootChildNodes;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT TreeNodeSortFilterProxyModel
: public QSortFilterProxyModel
, virtual public TreeNodeModelExtension
{

    Q_OBJECT

public:

    enum SpecialFilterType { UndefinedFilterType             = 0x0
                           , InvisibleFilterType             = 0x1
                           , SelectableFilterType            = 0x2 };


    TreeNodeSortFilterProxyModel( int /*specialFilters*/ = TreeNodeSortFilterProxyModel::UndefinedFilterType
                                , QObject * /*parent*/ = 0 );
    TreeNodeSortFilterProxyModel( QSet< int > /*acceptColumns*/
                                , int /*specialFilters*/ = TreeNodeSortFilterProxyModel::UndefinedFilterType
                                , QObject * /*parent*/ = 0 );
    virtual ~TreeNodeSortFilterProxyModel();

    const QSet<int> &acceptColumns() const;
    void setAcceptColumns( const QSet<int> &acceptColumns );

    virtual TreeNode * rootNode() const;

    virtual TreeNode * toTreeNode( const QModelIndex & /*index*/ ) const;
    virtual TreeNode * toTreeNode( const QItemSelectionRange & /*selectionRange*/ ) const;
    virtual TreeNode * toTreeNode( const QItemSelection & /*selection*/ ) const;

    virtual QModelIndex treeNodeToIndex( TreeNode * /*treeNode*/ ) const;

protected:

    virtual bool filterAcceptsColumn( int /*srcColumn*/, const QModelIndex & /*srcParentIndex*/ ) const;
    virtual bool filterAcceptsRow( int /*srcRow*/, const QModelIndex & /*srcParentIndex*/ ) const;

protected:

    int _specialFilters;
    QSet< int > _acceptColumns;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT FilterTreeNodeSortFilterProxyModel : public TreeNodeSortFilterProxyModel
{
    Q_OBJECT

public:

    FilterTreeNodeSortFilterProxyModel( int /*specialFilters*/ = TreeNodeSortFilterProxyModel::UndefinedFilterType
                                      , QObject * /*parent*/ = 0 );
    FilterTreeNodeSortFilterProxyModel( QSet< int > /*acceptsColumn*/
                                      , int /*specialFilters*/ = TreeNodeSortFilterProxyModel::UndefinedFilterType
                                      , QObject * /*parent*/ = 0 );
    virtual ~FilterTreeNodeSortFilterProxyModel();

protected:

    virtual bool filterAcceptsRow( int /*srcRow*/, const QModelIndex & /*srcParentIndex*/ ) const;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ChoiceTreeNodeProxyModel
: public QIdentityProxyModel
, virtual public TreeNodeModelExtension
{

    Q_OBJECT

signals:

    void stateChanged();

public:

    ChoiceTreeNodeProxyModel( int /*nameColumn*/
                            , int /*interceptChoiceRole*/
                            , int /*interceptBlockRole*/
                            , QObject * /*parent*/ = 0 );
    virtual ~ChoiceTreeNodeProxyModel();

    int nameColumn() const;
    int choiceRole() const;
    int blockRole() const;

    virtual QVariant data(const QModelIndex & /*index*/, int /*role*/ = Qt::DisplayRole) const;
    virtual bool setData(const QModelIndex & /*index*/, const QVariant & /*value*/, int /*role*/ = Qt::EditRole );
    virtual Qt::ItemFlags flags(const QModelIndex & /*index*/ ) const;

    void setCheckedBlockedNames( const QSet< QString > /*checkedNames*/
                               , const QSet< QString > /*blockedNames*/ );
    void setCheckedNames( const QSet< QString > /*checkedNames*/ );
    void setBlockedNames( const QSet< QString > /*blockedNames*/ );

    const QSet<QString> &checkedNames() const;
    const QSet<QString> &blockedNames() const;

    const QSet<QString> &addNames() const;
    const QSet<QString> &removeNames() const;
    void clearAddRemoveNames();


    virtual TreeNode * rootNode() const;

    virtual TreeNode * toTreeNode( const QModelIndex & /*index*/ ) const;
    virtual TreeNode * toTreeNode( const QItemSelectionRange & /*selectionRange*/ ) const;
    virtual TreeNode * toTreeNode( const QItemSelection & /*selection*/ ) const;

    virtual QModelIndex treeNodeToIndex( TreeNode * /*treeNode*/ ) const;

protected:

    int _nameColumn;
    int _choiceRole;
    int _blockRole;

    QSet< QString > _checkedNames;
    QSet< QString > _blockedNames;

    QSet< QString > _addNames;
    QSet< QString > _removeNames;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ChoiceTreeNodeSortFilterProxyModel
: public TreeNodeSortFilterProxyModel
{

    Q_OBJECT

public:

    ChoiceTreeNodeSortFilterProxyModel( bool /*acceptValue*/
                                      , QObject * /*parent*/ = 0 );

    virtual ~ChoiceTreeNodeSortFilterProxyModel();

protected:

    virtual bool filterAcceptsRow( int /*srcRow*/, const QModelIndex & /*srcParentIndex*/ ) const;

protected:

    bool _acceptValue;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ChoiceFilterTreeNodeSortFilterProxyModel
: public FilterTreeNodeSortFilterProxyModel
{

    Q_OBJECT

public:

    ChoiceFilterTreeNodeSortFilterProxyModel( bool /*acceptValue*/
                                            , QObject * /*parent*/ = 0 );

    virtual ~ChoiceFilterTreeNodeSortFilterProxyModel();

protected:

    virtual bool filterAcceptsRow( int /*srcRow*/, const QModelIndex & /*srcParentIndex*/ ) const;

protected:

    bool _acceptValue;

};

#endif // TREENODEMODEL_H
